package ke;

import java.util.*;


/**
 * MSc Knowledge Engineering 2004-2005 (Semester 2)
 * School of Informatics, The University of Edinburgh
 * Second Practical Assignment
 * Author: Michael Rovatsos
 */


//central agent used to maintain the message 
//blackboard
public class SystemManager implements Runnable {


    private List agents;     //agents running on the system
    private List blackboard; //the blackboard of agent messages
    
    //set this to true when you want the system to terminate
    private boolean terminate;


    //constructor, currently obtains only the number of agents to
    //construct as an argument
    public SystemManager(int agentNum) {
     blackboard = Collections.synchronizedList(new LinkedList());
     agents = new LinkedList();

     //start the system manager in its own thread of control
     (new Thread(this)).start();

     //generate agents (agent constructor will automatically spawn a new
     //thread for each agent)
     for(int i=0; i<agentNum; i++) 
	 agents.add(new Agent("agent"+i,this));
    }

    
    //this can be used by agents to post a message to the blackboard
    //that is visible for everyone
    public final void post(Message m) {
	blackboard.add(m);	
	System.out.println("New message: "+m);
    }


    //main processing loop for the system manager's thread
    //the manager thread will terminate as soon as evaluateContents()
    //has set the "terminate" variable to true
    //NOT TO BE MODIFIED
    public final void run() {
	while(!terminate) {
	    try{
		Thread.sleep(10);
	    }
	    catch(InterruptedException e) {
		e.printStackTrace();
	    }
	    evaluateContents();
	}
	Iterator it = agents.iterator();
	while(it.hasNext()) {
	    ((Agent) it.next()).terminate();
	}	
    }

    //method used to process and evaluate the blackboard contents
    //and to determine whether the termination criterion is met
    //(current dummy implementation simply sets the "terminate"
    //variable to true after 100000 messages have been received)
    private final void evaluateContents() {
	//modify blackboard to manage withdrawal of hypotheses (and
	//accept/reject statements referring to them), etc.

	//check whether termination criterion is met and set terminate to
	//true when it does so that the simulation is discontinued

	//current dummy criterion: stop after 1000 messages have been received
	if(blackboard.size()>1000) terminate=true; else terminate = false;

	//output solution
    }

    //method used by agents to access blackboard contents
    public final List getBlackboard() {
	return blackboard;
    }

}
 